@@ -462,8 +462,7 @@ def read_list(t, delim, i=0, module=None):
     obj_arr = False
     i = skip(t,i,ignore_newline=True)
     while not cmatch(t,i,delim) and i < len(t):
-        # TODO: make cleaner: kind of a hack to pass in read_neg but
-        #       we can knowingly read neg numbers in list context
+        # we can knowingly read neg numbers in list context
         i, q = kg_read(t, i, read_neg=True, ignore_newline=True, module=module)
         if q is None:
             break
@@ -475,7 +474,14 @@ def read_list(t, delim, i=0, module=None):
     if cmatch(t,i,delim):
         i += 1
     try:
-        return i, np.asarray(arr,dtype=object) if obj_arr else np.asarray(arr)
+        if obj_arr:
+            aa = np.asarray(arr,dtype=object)
+            # see if we can cast the array to a real type
+            if len(aa.shape) > 1: #jagged array is 1D (x,)
+                aa = np.asarray(arr)
+        else:
+            aa = np.asarray(arr)
+        return i, aa
     except ValueError:
         return i,cast_malformed_array(arr)
 
@@ -843,25 +849,29 @@ def merge_projections(arr):
     return sparse_fa
 
 
-def get_fn_arity(f, level=0):
+def get_fn_arity(f):
     """
     Examine a function AST and infer arity by looking for x,y and z.
     This arity is needed to populate the KGFn.
 
     NOTE: TODO: it maybe easier / better to do this at parse time vs late.
     """
-    if isinstance(f,KGFn):
-        x = get_fn_arity(f.a, level=1)
-        if is_list(f.args):
-            for q in f.args:
-                x.update(get_fn_arity(q,level=1))
-    elif is_list(f):
-        x = set()
-        for q in f:
-            x.update(get_fn_arity(q,level=1))
-    elif isinstance(f,KGSym):
-        x = set([f]) if f in reserved_fn_symbols else set()
-    else:
-        x = set()
-    return x if level else len(x)
-
+    if isinstance(f,KGFn) and isinstance(f.a,KGSym) and not in_map(f.a,reserved_fn_symbols):
+        # return f.arity - sum([1 for x in f.args if (x is not None) and not in_map(x,reserved_fn_symbols)])
+        return len({x for x in f.args if in_map(x, reserved_fn_symbols)}) + sum([1 for x in f.args if (x is None)])
+    def _e(f, level=0):
+        if isinstance(f,KGFn):
+            x = _e(f.a, level=1)
+            if is_list(f.args):
+                for q in f.args:
+                    x.update(_e(q,level=1))
+        elif is_list(f):
+            x = set()
+            for q in f:
+                x.update(_e(q,level=1))
+        elif isinstance(f,KGSym):
+            x = set([f]) if f in reserved_fn_symbols else set()
+        else:
+            x = set()
+        return x if level else len(x)
+    return _e(f)
