@@ -444,27 +444,42 @@ class KlongInterpreter():
         f_arity = x.arity
         f_args = [(x.args if isinstance(x.args, list) else [x.args]) if x.args is not None else x.args]
 
-        if self._context.is_defined_sym(f) or (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)):
-            while self._context.is_defined_sym(f) or (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)):
-                if (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)):
-                    f_args.append(f.args)
-                    f_arity = f.arity
-                    f = f.a
-                f = self.eval(f)
-                if isinstance(f,KGFn):
+        if isinstance(f,KGSym) and not in_map(f, reserved_fn_symbols):
+            f = self.eval(f)
+        if f_arity > 0:
+            if isinstance(f,KGFn) and not (f.is_op() or f.is_adverb_chain()) and ((f.args is None) or has_none(f.args) or (f_arity == 1 and has_none(f_args))):
+                if f.args is not None:
+                    f_args.append((f.args if isinstance(f.args, list) else [f.args]))
+                f_arity = f.arity
+                f = f.a
+            # don't resolve further if the underlying fn isn't a projection of greater arity
+            sub_resolve = (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)) and (f.arity > f_arity) and f.args is not None
+            resolve = sub_resolve or (self._context.is_defined_sym(f) and not in_map(f, reserved_fn_symbols))
+            if resolve:
+                if isinstance(f,KGSym) and not in_map(f, reserved_fn_symbols):
+                    f = self.eval(f)
+                if isinstance(f,KGFn) and not (f.is_op() or f.is_adverb_chain()) and ((f.args is None) or has_none(f.args) or (f_arity == 1 and has_none(f_args))):
                     if f.args is not None:
                         f_args.append((f.args if isinstance(f.args, list) else [f.args]))
                     f_arity = f.arity
                     f = f.a
-            f_args.reverse()
-        elif isinstance(f,KGFn) and not (f.is_op() or f.is_adverb_chain()):
-            # TODO: should this be recursive towards convergence?
-            if f.args is not None:
-                f_args.append(f.args)
-            f_args.reverse()
-            f_arity = f.arity
-            f = f.a
+                # don't resolve further if the underlying fn isn't a projection of greater arity
+                sub_resolve = (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)) and (f.arity > f_arity) and f.args is not None
+                resolve = sub_resolve or (self._context.is_defined_sym(f) and not in_map(f, reserved_fn_symbols))
+                if resolve:
+                    if isinstance(f,KGSym) and not in_map(f, reserved_fn_symbols):
+                        f = self.eval(f)
+                    if isinstance(f,KGFn) and not (f.is_op() or f.is_adverb_chain()) and ((f.args is None) or has_none(f.args) or (f_arity == 1 and has_none(f_args))):
+                        if f.args is not None:
+                            f_args.append((f.args if isinstance(f.args, list) else [f.args]))
+                        f_arity = f.arity
+                        f = f.a
+                    # don't resolve further if the underlying fn isn't a projection of greater arity
+                    # sub_resolve = (isinstance(f,KGFn) and self._context.is_defined_sym(f.a)) and (f.arity > f_arity)
+                    # resolve = sub_resolve or (self._context.is_defined_sym(f) and not in_map(f, reserved_fn_symbols))
 
+
+        f_args.reverse()
         f_args = merge_projections(f_args)
         if (f_args is None and f_arity > 0) or (is_list(f_args) and len(f_args) < f_arity) or has_none(f_args):
             return x
